/**********************************************************


                             CDROM functions
                        -------------------------

void Set(HWND _hwnd, HWND _hmenu, HAB _hab)
     set handle to window, menu, and anchor block
void Set_MM()
     set internal time format to MMTIME
void SetPositionAdvise()
     request periodic notifications during playback
---------------------------------------------------
BOOL Open()
     opens the CD-ROM device
void Acquire()
     acquire the CD device
void Close()
     closes the CD-ROM device
---------------------------------------------------
void Play()
     play the given track
void Stop()
     stop playback
void Pause()
     pause playback
void Resume()
     resume playback
void Previous()
     previous track
void Next()
     next track
void Insert()
     close the tray door
void Eject()
     open the tray door
---------------------------------------------------
void Set_SeekBackward(BOOL backward)
     set the direction of seeking
void Seeking(BOOL state)
     start or stop seeking
void Seek()
     seek for one time unit (1 second)
---------------------------------------------------
Struct_Volume Get_Volume()
     return the current volume level
void Set_Volume(Struct_Volume _volume)
     set the volume for the left and right channels
void Mute()
     mute the volume
void Unmute()
     unmute the volume
---------------------------------------------------
void DetectCD()
     detect if there's a CD present
void Get_capacities()
     get the device's capacities
void Get_TOC()
     query the Table Of Contents
---------------------------------------------------
void TOCtoa(char *text)
     convert the TOC to ascii
void mmtoa(MMTIME mm, char *time)
     convert MMTIME to ascii
---------------------------------------------------
int Get_CurrentTrackNumber()
     returns the current track number
---------------------------------------------------
void Set_TimeDisplay(int display)
     sets the format for the time display
int Get_TimeDisplay()
     returns the format for the time display
MMTIME Get_CurrentTime()
     returns the current time in MMTIME format
void Set_CurrentTime(MMTIME time)
     sets the current time
MMTIME Get_CorrectedededTime(MMTIME current)
     returns the time according to the specified time display
---------------------------------------------------
void Set_Connector(int _connector)
     set the connector (headphones or stream)
int Get_Connector()
     return the currently selected connector
---------------------------------------------------
---------------------------------------------------
BOOL UseOfCD
     Do we have use of the CD?
Struct_DeviceCapacities capacities
     capacities of the device



                         Example
                         -------

-create an instance of the class
     Class_CDROM cd;
-set the handles
     cd.Set(hwnd, hmenu, hab);
-open the device
     if(!cd.Open())
          ErrorBox(cd.message);
-close the device (when you're done with the device)
     cd.close();
-acquire the device
     cd.Acquire();
----------------------------------------------------
-execute basic commands
     cd.Play();
     cd.Stop();
     cd.Pause();
     cd.Resume();
     cd.Previous();
     cd.Next();
     cd.Insert();
     cd.Eject();
----------------------------------------------------
-seek backward
     cd.Set_SeekBackward(TRUE);
     cd.Seeking(TRUE);
     cd.Seek();
       ...
     cd.Seek();
     cd.Seeking(FALSE);
-seek forward
     cd.Set_SeekBackward(FALSE);
     cd.Seeking(TRUE);
     cd.Seek();
       ...
     cd.Seek();
     cd.Seeking(FALSE);
----------------------------------------------------
-obtain the current software volume level
     Struct_Volume volume;
     volume = cd.Get_Volume();
-set the volume level (from 0 to 100) for the left and right channels
     Struct_Volume volume;
     volume.left = 50;
     volume.right = 100;
     cd.Set_Volume(volume);
-mute the volume
     cd.Mute();
-unmute the volume
     cd.Unmute();
-detect if there's an audio CD in the device
     cd.DetectCD();
-obtain the device's capacities
     cd.Get_Capacities();
-query the audio CD's Table of Contents
     cd.Get_TOC();
----------------------------------------------------
-convert the Table of Contents to an ascii string
     char text[500];
     cd.TOCtoa(text);
-convert a time from the MMTIME format to an ascii string
     MMTIME mmtime;
     char time[9];
     cd.mmtoa(mmtime, time);
----------------------------------------------------
-query the number of the current track
     int current_track;
     current_track = cd.Get_CurrentTrackNumber();
----------------------------------------------------
-set the time display
     cd.Set_TimeDisplay(IDM_TIME_TRACKPLAYED);
-query the currently set time display
     int time_display;
     time_display = cd.Get_TimeDisplay();
-obtain the current time in MMTIME format
     MMTIME current_time;
     current_time = cd.Get_CurrentTime();
-set the current time in MMTIME format
     MMTIME current_time = 0;
     cd.Set_CurrentTime(current_time);
-transform the specified time to a representation conforming to the current time display
     MMTIME mmtime = 0;
     MMTIME corrected_time;
     corrected_time = cd.Get_CorrectedTime(mmtime);
----------------------------------------------------
-set the connector to use (headphones)
     cd.Set_Connector(MCI_HEADPHONES_CONNECTOR);
-set the connector to use (stream)
     cd.Set_Connector(MCI_CD_STREAM_CONNECTOR);
-query the currently used connector
     int connector;
     cd.Get_Connector();
----------------------------------------------------
----------------------------------------------------
-determine if we have use of the CD
     if(UseOfCd) {
          // We have use of the CD
     }
     if(!UseOfCD) {
          // We do not have use of the CD
     }
-query a certain capacity of the device
 (examine the Struct_DeviceCapacities structure for possible members)
     if(capacities.Play) {
          // The device can play
     }


**********************************************************/

#ifndef CLASS_CDROM_HPP
#define CLASS_CDROM_HPP

// Structure for the volume
struct Struct_Volume {
     int left;// left volume
     int right;// right volume
};

// Structure for the Table of Contents
struct Struct_TOC {
     MCI_TOC_PARMS tocHeader;// Table of Contents header
     MCI_TOC_REC *record;// Table of Contents entries
     int track_count;// number of tracks on this audio CD
     char *Title;// title of the CD (unused)
     char **track;// pointer to a table of track titles (unused)
};

// Structure for the device's capacities
struct Struct_DeviceCapacities {
     BOOL Eject; // can eject the media
     BOOL LockEject;// can prevent manual ejection
     BOOL Play;// can play, stop, pause, and resume
     BOOL ProcessInternal;// internally process digital data
     BOOL Record;// can record
     BOOL RecordInsert;// insert data while recording
     BOOL Save;// can save files
     BOOL SetVolume;// can change audio volume level
     BOOL StreamDigital;// can stream digital data continously
     ULONG DeviceType;// ID of this device type
     BOOL Audio;// can play audio
     BOOL Image;// can display a still image
     BOOL Video;// can play video
     // Query the number of connectors of each type
     int Headphones;
     int Stream;
};

#include "Class_Controls.hpp"

class Class_CDROM : public Class_Controls {
public:
     Class_CDROM();// constructor
     ~Class_CDROM();// destructor
     void Set(HWND _hwnd, HWND _hmenu, HAB _hab);
     void Set_MM();
     //----------------------------------------------------
     BOOL Open();
     void Acquire();
     void Close();
     //----------------------------------------------------
     void Play();
     void Stop();
     void Pause();
     void Resume();
     void Previous();
     void Next();
     void Insert();
     void Eject();
     //----------------------------------------------------
     void Set_SeekBackward(BOOL backward);
     void Seeking(BOOL state);
     void Seek();
     //----------------------------------------------------
     void Query_Volume();
     Struct_Volume Get_Volume();
     void Set_Volume(Struct_Volume _volume);
     void Mute();
     void Unmute();
     //----------------------------------------------------
     void DetectCD();
     void Get_Capacities();
     void Get_TOC();
     //----------------------------------------------------
     void TOCtoa(char *text);
     void mmtoa(MMTIME mm, char *time);
     //----------------------------------------------------
     int Get_CurrentTrackNumber();
     MMTIME PositionInTrack();
     //----------------------------------------------------
     void Set_TimeDisplay(int display);
     int Get_TimeDisplay();
     MMTIME Get_CurrentTime();
     void Set_CurrentTime(MMTIME time);
     MMTIME Get_CorrectedTime(MMTIME time);
     //----------------------------------------------------
     void Set_Connector(int _connector);
     int Get_Connector();
     //----------------------------------------------------
     //----------------------------------------------------
     BOOL UseOfCD;// Do we have use of the CD?
     Struct_DeviceCapacities capacities;// capacities of the device
private:
     BOOL ProcessReturnCode(ULONG rc, char *error_message);
     //----------------------------------------------------
     void EnablePlay(BOOL state);
     void ActivatePlay(BOOL state);
     void EnablePause(BOOL state);
     void ActivatePause(BOOL state);
     void ActivateBackward(BOOL state);
     void ActivateForward(BOOL state);
     void ActivatePrevious(BOOL state);
     void ActivateNext(BOOL state);
     void EnableEject(BOOL state);
     void EnableMute(BOOL state);
     void ActivateMute(BOOL state);
     //----------------------------------------------------
     void SeekTo(MMTIME time);
     void SetPlayPosition();
     void SetPositionAdvise();
     void RestartCurrentAction();
     void RemoveCuePoint();
     void SetCuePoint();
     void PreviousTrack();
     void NextTrack();
     //----------------------------------------------------
     //----------------------------------------------------
     HWND hwnd;// handle to the window
     HWND hmenu;// handle to the menu
     HAB hab;// handle to anchor block
     ULONG ulrc;// return code
     char *message;// error message after a command fails
     unsigned short usDeviceID;// ID of the CDROM device
     unsigned short VolumeDeviceID;// ID of the device in charge of volume level
     ULONG old_cue_point;// previous cue point
     BOOL playing;// is the CD playing?
     BOOL paused;// is the CD paused?
     ULONG timerID;// ID of the timer used when seeking
     int current_track;// number of the current track
     int last_track;// number of the last track
     MMTIME mmtime;// current time in mmtime format
     BOOL seeking;// are we seeking?
     BOOL SeekBackward;// TRUE if seeking backward
                       // FALSE if seeking forward
     int TimeFormat;// Currently selected time format
     Struct_TOC TOC;// table of content of the CD
     Struct_Volume volume;// left and right volume level
     int connector;// headphones or stream

     // various MCI structures
     MCI_OPEN_PARMS mciOpenParameters;
     MCI_PLAY_PARMS mciPlayParameters;
     MCI_GENERIC_PARMS mciGenericParameters;
     MCI_SET_PARMS mciSetParameters;
     MCI_SEEK_PARMS mciSeekParameters;
     MCI_STATUS_PARMS mciStatusParameters;
     MCI_CUEPOINT_PARMS mciCuepointParameters;
     MCI_GETDEVCAPS_PARMS mciGetDevCapsParameters;
     MCI_POSITION_PARMS mciPositionParameters;
     MCI_CONNECTOR_PARMS mciConnectorParameters;
     MCI_CONNECTORINFO_PARMS mciConnectorInfoParameters;
     MCI_CONNECTION_PARMS mciConnectionParameters;
     MCI_MASTERAUDIO_PARMS mciMasteraudioParameters;
};

//---------------------------------------------------------
// inline functions

inline Class_CDROM::Class_CDROM() {
     // Constructor

     // Initialize variables
     hwnd = hmenu = NULL;
     hab = NULL;
     current_track = last_track = 0;
     old_cue_point = 0;
     TOC.record = new MCI_TOC_REC;// prepare the TOC array
     playing = paused = FALSE;// we're not doing these actions
     mmtime = 0;// current time
     timerID = 1;// ID of the timer
     seeking = FALSE;// we're not seeking
     UseOfCD = TRUE;
     connector = 0;// there's no connector at this time
     VolumeDeviceID = 0;
     message = new char [500];
     if(!message) {
          ErrorBox("Allocation error for message");
          exit(1);
     }
}// Class_CDROM()

//---------------------------------------------------------

inline Class_CDROM::~Class_CDROM() {
     // Destructor

     // Release allocated memory
     delete [] TOC.record;
     delete [] message;
}// ~Class_CDROM()

//---------------------------------------------------------

inline void Class_CDROM::Set(HWND _hwnd, HWND _hmenu, HAB _hab) {
     // Set various data

     // Set handles
     hwnd = _hwnd;
     hmenu = _hmenu;
     hab = _hab;

     // Set handles for Class_Controls
     Class_Controls::Set(hwnd, hmenu);

     // Specify the handle to the window that will receive
     // the notification when the command is completed.
     // Even if we do not use MCI_NOTIFY for most commands,
     // some nevertheless require that this be set.
     mciOpenParameters.hwndCallback = hwnd;
     mciPlayParameters.hwndCallback = hwnd;
     mciGenericParameters.hwndCallback = hwnd;
     mciSetParameters.hwndCallback = hwnd;
     mciSeekParameters.hwndCallback = hwnd;
     mciStatusParameters.hwndCallback = hwnd;
     mciCuepointParameters.hwndCallback = hwnd;
     mciGetDevCapsParameters.hwndCallback = hwnd;
     mciPositionParameters.hwndCallback = hwnd;
     mciConnectorParameters.hwndCallback = hwnd;
     mciConnectorInfoParameters.hwndCallback = hwnd;
     mciConnectionParameters.hwndCallback = hwnd;
     // mciMasterAudioParameters does not have an hwdnCallback member
}

//---------------------------------------------------------

inline void Class_CDROM::Set_SeekBackward(BOOL backward) {
     // Set the direction of seeking
     SeekBackward = backward;
}// Set_SeekBackward()

//---------------------------------------------------------

inline Struct_Volume Class_CDROM::Get_Volume() {
     // return the current software values of the volume
     return volume;
}// Get_Volume()

//---------------------------------------------------------

inline int Class_CDROM::Get_CurrentTrackNumber() {
     // Returns the current track

     return TOC.record[current_track].TrackNum;
}// Get_CurrentTrackNumber()

//---------------------------------------------------------

inline void Class_CDROM::Set_TimeDisplay(int display) {
     // Set the type of time display

     TimeFormat = display;
}// Set_TimeDisplay()

//---------------------------------------------------------

inline int Class_CDROM::Get_TimeDisplay() {
     // Return the selected time display

     return TimeFormat;
}// Get_TimeDisplay()

//---------------------------------------------------------

inline MMTIME Class_CDROM::Get_CurrentTime() {
     // Return the current time

     return mmtime;
}// Get_CurrentTime()

//---------------------------------------------------------

inline void Class_CDROM::Set_CurrentTime(MMTIME time) {
     // Set the current time

     mmtime = time;
}// Set_CurrentTime()

//---------------------------------------------------------

inline int Class_CDROM::Get_Connector() {
     // Return the current connector

     return connector;
}// Get_Connector()

//---------------------------------------------------------

inline BOOL Class_CDROM::ProcessReturnCode(ULONG rc, char *error_message) {
     // Processes the return code

     // If a message other than MCIERR_SUCCESS is passed,
     // its textual description will be placed in the
     // message variable and FALSE will be returned

     if(LOUSHORT(rc) == MCIERR_SUCCESS) {
          strcpy(message, "");
          return TRUE;
     }
     else {
          char text[500];// To contain the MCI error message
          mciGetErrorString(rc,
                            text,
                            sizeof(text));
          sprintf(message, "%s\n%s", text, error_message);
          return FALSE;
     }
}// ProcessReturnCode()

//---------------------------------------------------------

inline void Class_CDROM::EnablePlay(BOOL state) {
     // Enable/disable the Play and Stop actions

     EnableMenuitem(IDM_PLAY, state);
     ShowButton(IDD_PB_PLAY, state);
     EnableMenuitem(IDM_STOP, !state);
     ShowButton(IDD_PB_STOP, !state);
     playing = !state;// not obvious, but this is correct
}// EnablePlay()

//---------------------------------------------------------

inline void Class_CDROM::ActivatePlay(BOOL state) {
     // Activate/deactivate the Play and stop actions

     EnableMenuitem(IDM_PLAY, state);
     WinEnableControl(hwnd, IDD_PB_PLAY, state);
}// ActivatePlay()

//---------------------------------------------------------

inline void Class_CDROM::EnablePause(BOOL state) {
     // Enable/disable the Pause and Resume actions

     EnableMenuitem(IDM_PAUSE, state);
     ShowButton(IDD_PB_PAUSE, state);
     EnableMenuitem(IDM_RESUME, !state);
     ShowButton(IDD_PB_RESUME, !state);
     paused = !state; // not obvious, but this is correct
}// EnablePause()

//---------------------------------------------------------

inline void Class_CDROM::ActivatePause(BOOL state) {
     // Activate/deactivate the Pause action

     EnableMenuitem(IDM_PAUSE, state);
     WinEnableControl(hwnd, IDD_PB_PAUSE, state);
}// ActivatePause()

//---------------------------------------------------------

inline void Class_CDROM::ActivateBackward(BOOL state) {
     // Activate/deactivate the Backward action

     EnableMenuitem(IDM_BACKWARD, state);
     WinEnableControl(hwnd, IDD_GB_BACKWARD, state);
}// ActivateBackward()

//---------------------------------------------------------

inline void Class_CDROM::ActivateForward(BOOL state) {
     // Activate/deactivate the Forward action

     EnableMenuitem(IDM_FORWARD, state);
     WinEnableControl(hwnd, IDD_GB_FORWARD, state);
}// EnableForward()

//---------------------------------------------------------

inline void Class_CDROM::ActivatePrevious(BOOL state) {
     // Activate/deactivate the Previous action

     EnableMenuitem(IDM_PREVIOUS, state);
     WinEnableControl(hwnd, IDD_PB_PREVIOUS, state);
}// EnablePrevious()

//---------------------------------------------------------

inline void Class_CDROM::ActivateNext(BOOL state) {
     // Activate/deactivate the Next action

     EnableMenuitem(IDM_NEXT, state);
     WinEnableControl(hwnd, IDD_PB_NEXT, state);
}// EnableNext()

//---------------------------------------------------------

inline void Class_CDROM::EnableEject(BOOL state) {
     // Activate/deactivate the Eject and Insert actions
     // as well as enabling/disabling the Dectec CD
     // menuitem

     EnableMenuitem(IDM_EJECT, state);
     ShowButton(IDD_PB_EJECT, state);
     EnableMenuitem(IDM_INSERT, !state);
     ShowButton(IDD_PB_INSERT, !state);
     EnableMenuitem(IDM_DETECTCD, state);
}// EnableEject()

//---------------------------------------------------------

inline void Class_CDROM::EnableMute(BOOL state) {
     // Enable/disable the Mute and Unmute actions

     EnableMenuitem(IDM_MUTE, state);
     ShowButton(IDD_PB_MUTE, state);
     EnableMenuitem(IDM_UNMUTE, !state);
     ShowButton(IDD_PB_UNMUTE, !state);
}// EnableMute()

//---------------------------------------------------------

inline void Class_CDROM::ActivateMute(BOOL state) {
     // Activate/deactivate the Mute action

     EnableMenuitem(IDM_MUTE, state);
     WinEnableControl(hwnd, IDD_PB_MUTE, state);
}// ActivateMute()

//---------------------------------------------------------

inline void Class_CDROM::PreviousTrack() {
     // Determine the previous track to be played

     // In this implementation, we go to the beginning
     // of the current track if we've played less than
     // half a second within that track, otherwise we go
     // to the preceeding track or to the last track if
     // we've passed the first track

     if(MSECFROMMM(PositionInTrack()) < 500) {
          // Are we further than 500 msec in the current track?
               // Go to the previous track
               current_track--;
               if(current_track == -1)
                    current_track = last_track - 1;

               // Since we've changed track, we need
               // to set a new cuepoint
               SetCuePoint();
               WinSendMsg(hwnd, WM_USER_SHOW_TRACK, 0, 0);
          }
          else {
               // We stay in the current track.
               // There's nothing special to do since the
               // function SetPlayPosition works by taking
               // the current track and going to its
               // beginning.
          }
}// PreviousTrack()

//---------------------------------------------------------

inline void Class_CDROM::NextTrack() {
     // Determine the next track to be played

     // In this implementation, we go to the next track
     // or the first track if we've passed the last track
     current_track++;
     if(current_track == last_track)
          current_track = 0;

     // Since we've changed track, we need to set a
     // new cuepoint
     SetCuePoint();
     WinSendMsg(hwnd, WM_USER_SHOW_TRACK, 0, 0);
}// NextTrack();

//---------------------------------------------------------

#endif // CLASS_CDROM_HPP
