/**********************************************************

            Common actions on menus and buttons
            -----------------------------------

void Set(HWND _hwnd, HWND _hmenu)
     Set handle to window and menu
void CheckMenuitem(int menuitem, BOOL state)
     Check or uncheck a menuitem
void EnableMenuitem(int menuitem, BOOL state)
     Enable or disable a menuitem
void ShowButton(int button, BOOL state)
     Show or hide a button


                           Example
                           -------

-create an object
     Class_Controls controls
 or 'add' to an object
     class Class_Object : public Class_Controls {
          // definition of Class_Object
     };
-initialise
     controls.set(hwnd, hmenu);
-check a menuitem
     controls.CheckMenuitem(IDM_MENUITEM, TRUE);
-uncheck a menuitem
     controls.CheckMenuitem(IDM_MENUITEM, FALSE);
-enable a menuitem
     controls.EnableMenuitem(IDM_MENUITEM, TRUE);
-disable a menuitem
     controls.EnableMenuitem(IDM_MENUITEM, FALSE);
-show a button
     controls.ShowButton(IDD_PB_BUTTON, TRUE);
-hide a button
     controls.ShowButton(IDD_PB_BUTTON, FALSE);



**********************************************************/

#ifndef CLASS_CONTROLS_HPP
#define CLASS_CONTROLS_HPP

#define INCL_PM
#include "Basic.hpp"

#include <os2.h>
#include "Resource.hpp"

class Class_Controls {
public:
     Class_Controls();
     void Set(HWND _hwnd, HWND _hmenu);
     //----------------------------------------------------
     void CheckMenuitem(int menuitem, BOOL state);
     void EnableMenuitem(int menuitem, BOOL state);
     void ShowButton(int button, BOOL state);
//*********************************************************
private:
     HWND hwnd;// handle to the window
     HWND hmenu;// handle to the menu
};

//---------------------------------------------------------
// inline functions

inline Class_Controls::Class_Controls() {
     // initialize
     hwnd = hmenu = NULL;
} // Class_Controls()

//---------------------------------------------------------

inline void Class_Controls::Set(HWND _hwnd, HWND _hmenu) {
     // Set handles
     hwnd = _hwnd;
     hmenu = _hmenu;
}// Set()

//---------------------------------------------------------

inline void Class_Controls::ShowButton(int button, BOOL state) {
     // show or hide a window

     WinShowWindow(WinWindowFromID(hwnd, button),
                   state);
}// ShowButton()

//---------------------------------------------------------

inline void Class_Controls::CheckMenuitem(int menuitem, BOOL state) {
     // check or uncheck a menuitem

     WinSendMsg(hmenu,
                MM_SETITEMATTR,
                MPFROM2SHORT(menuitem, TRUE),
                MPFROM2SHORT(MIA_CHECKED,
                state ? MIA_CHECKED : 0));
}// CheckMenuitem()

//---------------------------------------------------------

inline void Class_Controls::EnableMenuitem(int menuitem, BOOL state) {
     // enable or disable a menuitem

     WinSendMsg(hmenu,
                MM_SETITEMATTR,
                MPFROM2SHORT(menuitem, TRUE),
                MPFROM2SHORT(MIA_DISABLED,
                state ? 0: MIA_DISABLED));
};// EnableMenuitem()

#endif // CLASS_CONTROLS_HPP
