#define INCL_WIN
#define INCL_PM
#define INCL_DOSPROCESS
#define INCL_WINSWITCHLIST

#include <stdlib.h>
#include <stdio.h>
#include <process.h>
#include <string.h>
#include <os2.h>

#define INCL_OS2MM
#define INCL_MCIOS2
#define INCL_MMIOOS2
#define INCL_SW
#define INCL_GRAPHICBUTTON
#include <os2me.h>

#include "Basic.hpp"
#include "Resource.hpp"
#include "Class_CDROM.hpp"

struct WINDOWDATA {
     HWND hmenu;// handle to the menu
     SWCNTRL swctl;// used for tasklist
     HSWITCH hswitch;// handle to the tasklist entry
     PID pid;// program ID
     GBTNCDATA GraphicButton_Backward;// backward graphic button
     GBTNCDATA GraphicButton_Forward;// forward graphic button
     HAB hab;// handle to anchor block
     Class_CDROM cd;// class that helps out
     char time[9];// formatted time display
     char track[3];// formatted track display
};

//---------------------------------------------------------
// prototypic declarations

MRESULT EXPENTRY Dlg_Connector(HWND hwnd,
                               ULONG msg,
                               MPARAM mp1,
                               MPARAM mp2);

MRESULT EXPENTRY Dlg_Volume(HWND hwnd,
                            ULONG msg,
                            MPARAM mp1,
                            MPARAM mp2);

//---------------------------------------------------------
//---------------------------------------------------------

// This dialog handles all the messages from the user by
// calling upon the methods of the Class_CDROM class.
// It also handles notifications from various MCI commands.

MRESULT EXPENTRY Main_Dialog(HWND hwnd,
                             ULONG msg,
                             MPARAM mp1,
                             MPARAM mp2) {

WINDOWDATA *wd;// window data

switch(msg){

case WM_INITDLG:
     // allocate memory for the window words
     wd=new WINDOWDATA;
     if(!wd){
          ErrorBox("Critical error creating a window word in WM_INITDLG");
          WinDestroyWindow(hwnd);
          exit(1);
     }
     if(!WinSetWindowPtr(hwnd, 0, wd)){
          ErrorBox("Critical error setting window word 0  in WM_INITDLG");
          WinDestroyWindow(hwnd);
          exit(1);
     }

     // Add the dialog to the task list
     WinQueryWindowProcess(hwnd, &wd->pid, NULL);
     wd->swctl.hwnd = hwnd;
     wd->swctl.hwndIcon = NULLHANDLE;
     wd->swctl.hprog = NULLHANDLE;
     wd->swctl.idProcess = wd->pid;
     wd->swctl.idSession = 0;
     wd->swctl.uchVisibility = SWL_VISIBLE;
     wd->swctl.fbJump = SWL_JUMPABLE;
     strcpy(wd->swctl.szSwtitle, "CD Player");
     wd->swctl.bProgType = PROG_DEFAULT;
     wd->hswitch = WinAddSwitchEntry(&wd->swctl);

     // query the hab
     wd->hab = WinQueryAnchorBlock(hwnd);

     // setup the graphic buttons
     if(!WinRegisterGraphicButton()) {
          ErrorBox("Unable to register GraphicButton class");
          WinSendMsg(hwnd, WM_CLOSE, 0, 0);
     }
     // For each graphic button, set
     //    -the handle of the bitmap resource
     //    -the number of bitmaps
     //    -the ID of each of the bitmap (in an array)
     //          you'll need to allocate memory if there's
     //          more than one bitmap
     wd->GraphicButton_Backward.hmod=0;
     wd->GraphicButton_Backward.cBitmaps=1;
     wd->GraphicButton_Backward.aidBitmap[0] = IDD_GB_BACKWARD;
     wd->GraphicButton_Forward.hmod=0;
     wd->GraphicButton_Forward.cBitmaps=1;
     wd->GraphicButton_Forward.aidBitmap[0] = IDD_GB_FORWARD;

     // load the menu in the dialog
     wd->hmenu = WinLoadMenu(hwnd, 0, IDD_DIALOG_MAIN);
     WinSendMsg(hwnd, WM_UPDATEFRAME, 0, 0);

     // set data for Class_CD
     wd->cd.Set(hwnd, wd->hmenu, wd->hab);
     wd->cd.Set_TimeDisplay(IDM_TIME_TRACKPLAYED);

     if(!wd->cd.Open()) {
          // Unable to open the audio CD device
          WinSendMsg(hwnd, WM_CLOSE, 0, 0);
     }
     break;

//----------------------------------

case WM_CLOSE:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in WM_CLOSE");
     }

     wd->cd.Close();
     delete wd;// release allocated memory
     // This message is required to close the main dialog
     WinPostMsg(hwnd, WM_QUIT, 0, 0);
     break;

//----------------------------------

case WM_COMMAND:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in WM_COMMAND");
     }

     switch(SHORT1FROMMP(mp1)) {
     // message from a button or a menu
     //----------------------------------
     case IDM_CONNECTOR:
          // Open and process the connector dialog
          WinDlgBox(HWND_DESKTOP,
                    hwnd,
                    Dlg_Connector,
                    0,
                    IDD_DIALOG_CONNECTOR,
                    &wd->cd);
          break;
     //----------------------------------
     case IDM_LEVEL:
          // Open and process the volume dialog
          WinDlgBox(HWND_DESKTOP,
                    hwnd,
                    Dlg_Volume,
                    0,
                    IDD_DIALOG_VOLUME,
                    &wd->cd);
          break;
     //----------------------------------
     case IDM_PLAY:
     case IDD_PB_PLAY:
          wd->cd.Play();
          break;
     //----------------------------------
     case IDM_STOP:
     case IDD_PB_STOP:
          wd->cd.Stop();
          break;
     //----------------------------------
     case IDM_PAUSE:
     case IDD_PB_PAUSE:
          wd->cd.Pause();
          break;
     //----------------------------------
     case IDM_RESUME:
     case IDD_PB_RESUME:
          wd->cd.Resume();
          break;
     //----------------------------------
     case IDM_PREVIOUS:
     case IDD_PB_PREVIOUS:
          wd->cd.Previous();
          break;
     //----------------------------------
     case IDM_NEXT:
     case IDD_PB_NEXT:
          wd->cd.Next();
          break;
     //----------------------------------
     case IDM_BACKWARD:
     case IDM_FORWARD:
          InfoBox("This option is only availble through the buttons.");
          break;
     //----------------------------------
     case IDM_EJECT:
     case IDD_PB_EJECT:
          wd->cd.Eject();
          break;
     //----------------------------------
     case IDM_INSERT:
     case IDD_PB_INSERT:
          wd->cd.Insert();
          break;
     //----------------------------------
     case IDM_MUTE:
     case IDD_PB_MUTE:
          wd->cd.Mute();
          break;
     //----------------------------------
     case IDM_UNMUTE:
     case IDD_PB_UNMUTE:
          wd->cd.Unmute();
          break;
     //----------------------------------
     case IDM_TIME_CDREMAIN:
     case IDM_TIME_CDPLAYED:
     case IDM_TIME_TRACKREMAIN:
     case IDM_TIME_TRACKPLAYED:
          // uncheck the current selection
          wd->cd.CheckMenuitem(wd->cd.Get_TimeDisplay(),
                               FALSE);

          // set the current time display to the selection
          wd->cd.Set_TimeDisplay(SHORT1FROMMP(mp1));

          // check the newly selected time display
          wd->cd.CheckMenuitem(wd->cd.Get_TimeDisplay(),
                               TRUE);
          WinSendMsg(hwnd, WM_USER_SHOW_TIME, 0, 0);
          break;
     //----------------------------------
     case IDM_EXIT:
          WinPostMsg(hwnd, WM_CLOSE, 0, 0);
          break;
     //----------------------------------
     case IDM_TOC:
          {
          char text[1000];
          wd->cd.TOCtoa(text);
          WinMessageBox(HWND_DESKTOP,
                        HWND_DESKTOP,
                        (PSZ) text,
                        (PSZ) "Table of contents",
                        0,
                        MB_OK | MB_INFORMATION | MB_MOVEABLE);
          }
          break;
     //----------------------------------
     case IDM_DETECTCD:
          wd->cd.DetectCD();
          break;
     //----------------------------------
     case IDM_ABOUT:
          WinDlgBox(HWND_DESKTOP,
                    hwnd,
                    NULL,
                    0,
                    IDD_DIALOG_ABOUT,
                    0);
          break;

     }//switch(SHORT1FROMMP(mp1))
     return 0;

//----------------------------------

case WM_CONTROL:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in WM_CONTROL");
     }

     // One of the seek buttons is pressed or released
     switch(SHORT2FROMMP(mp1)){
          case GBN_BUTTONHILITE: // a button is held down
               // Determine the direction of seeking:
               //            TRUE:  backward
               //            FALSE: forward
               // depending on the button that was pressed
               wd->cd.Set_SeekBackward(SHORT1FROMMP(mp1) == IDD_GB_BACKWARD);
               wd->cd.Seeking(TRUE); // start seeking
               break;
          case GBN_BUTTONUP: // the pressed button is released
               wd->cd.Seeking(FALSE);// stop seeking
               break;
     }//switch(SHORT2FROMMP(mp1))
     return 0;

//----------------------------------

case WM_TIMER:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in WM_TIMER");
     }

     // Time to seek to the next time unit
     wd->cd.Seek();
     break;

//----------------------------------

case WM_USER_SHOW_TRACK:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in WM_USER_SHOW_TRACK");
     }

     // Display the current track
     sprintf(wd->track, "%02d", wd->cd.Get_CurrentTrackNumber());
     WinSetDlgItemText(hwnd, IDD_STATIC_TRACK, wd->track);
     WinSendMsg(hwnd, WM_UPDATEFRAME, 0, 0);
     break;

//----------------------------------

case WM_USER_SHOW_TIME:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in WM_USER_SHOW_TIME");
     }

     // Display the current time in the appropriate format
     wd->cd.mmtoa(wd->cd.Get_CorrectedTime(wd->cd.Get_CurrentTime()),
                  wd->time);
     WinSetDlgItemText(hwnd,
                       IDD_STATIC_TIME,
                       wd->time);
     WinSendMsg(hwnd, WM_UPDATEFRAME, 0, 0);
     break;

//----------------------------------

case MM_MCINOTIFY:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in MM_MCINOTIFY");
     }

     switch(SHORT1FROMMP(mp1)) { // usNotifycode
     case MCI_NOTIFY_SUCCESSFUL:
          // the command completed successfully
          switch(SHORT2FROMMP(mp2)) {
          // which message generated this notification
          case MCI_OPEN:
               // We've successfully opened the device
               // and are now ready for other tasks

               // Since these may take some time, we should
               // bring up the hourglass pointer (not done)

               // Query the device capacities
               wd->cd.Get_Capacities();

               // Set the volume level to the maximum by default
               {
                    Struct_Volume volume;
                    volume.left = volume.right = 100;
                    wd->cd.Set_Volume(volume);
               }

               // Detect if there's a CD
               wd->cd.DetectCD();

               // Now that we're done with these lengthy
               // tasks, we should remove the hourglass
               // pointer (not done)

               break;
          case MCI_PLAY:
               // This means that we've reached the end of
               // the last track

               // Here are two possible implementations.
               // In the first, we simply stop the CD
               // and go to the next track.  We go to the
               // next track because we'll also receive a
               // MM_MCICUEPOINT message
               wd->cd.Stop();

               // In the second, we play the next track
               // (There's nothing to do since the CD
               //  automatically goes to the next track
               //  when it reaches the end of the current
               //  track and generates a MM_MCICUEPOINT
               //  message.)  This would implement a
               // continuous play feature.
               break;
          default:
               // You used MCI_NOTIFY rather than MCI_WAIT
               // and forgot to catch the notification
               InfoBox("Unexpected MCI_NOTIFY_SUCCESSFUL");
               break;
          }
          break;
     case MCI_NOTIFY_SUPERSEDED:
          // Another request was received and will be
          // processed.  ex. a new MCI_PLAY message
          // supersedes a previous one
          break;
     case MCI_NOTIFY_ABORTED:
          // The command was interrupted by another
          // ex. MCI_STOP aborts MCI_PLAY.
          break;
     default:
          // An error occurred.
          // The command that generated the error message
          // is contained in short2 of mp2
          switch(SHORT2FROMMP(mp2)) {
          // And the ID of the error is contained in short1
          // of mp1
          case MCI_OPEN:
               // something's gone wrong with the command
               {
               char text[1000];
               mciGetErrorString(SHORT1FROMMP(mp1),
                                 text,
                                 sizeof(text));
               ErrorBox(text);
               }
               WinSendMsg(hwnd, WM_CLOSE, 0, 0);
               break;
          } // who generated this message
          break;
     } // usNotifycode
     break;

//----------------------------------

case MM_MCICUEPOINT:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in MM_MCICUEPOINT");
     }

     // We've reached the end of the current track
     // so it's now time to play the next track
     wd->cd.Next();
     break;

//----------------------------------

case MM_MCIPASSDEVICE:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in MM_MCIPASSDEVICE");
     }

     // We're losing or gaining the use of the device
     switch(SHORT1FROMMP(mp2)){
      case MCI_GAINING_USE:
          // We have use of the CD
          wd->cd.UseOfCD = TRUE;
          break;
      case MCI_LOSING_USE:
          // We've lost the use of the CD
          wd->cd.UseOfCD = FALSE;
          break;
     }
     break;

//----------------------------------

case WM_ACTIVATE:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in WM_CONTROL");
     }

     if((BOOL) mp1) {
          // The window has been activated
          if(!wd->cd.UseOfCD) {
               // and we do not have the use of the device
               wd->cd.Acquire();
          }
     }
     else {
          // The window is losing focus
     }
     break;

//----------------------------------

case MM_MCIPOSITIONCHANGE:
     // retrieve the window words
     wd=(WINDOWDATA *)WinQueryWindowPtr(hwnd, 0);
     if(!wd){
          ErrorBox("Error retrieving window word 0 in MM_MCIPOSITIONCHANGE");
     }

     // mp2 contains the current time
     // We'll use that to correctly keep track of where the
     // CD is located
     wd->cd.Set_CurrentTime((MMTIME) mp2);
     WinSendMsg(hwnd, WM_USER_SHOW_TIME, 0, 0);
     break;

}//switch(msg)

return WinDefDlgProc(hwnd, msg, mp1, mp2);

}//Basic Dialog Procedure

//---------------------------------------------------------
//---------------------------------------------------------

// This dialog lets the user select one of the available
// connectors.  Since we've already queried the device,
// we know which connectors are supported by the device,
// and we make only those available.  Also, I've elected
// to make the change of connector immediate rather than
// when the dialog is closed.  To enable that feature, I
// simply passed the Class_CDROM class object.

MRESULT EXPENTRY Dlg_Connector(HWND hwnd,
                               ULONG msg,
                               MPARAM mp1,
                               MPARAM mp2) {
static Class_CDROM *ptr;// pointer to the CDROM class object

switch(msg){
     case WM_INITDLG:
          // Retrieve the pointer to the Class_CDROM object
          ptr = (Class_CDROM *) mp2;

          switch(ptr->Get_Connector()) {
          // Select the active connector
          case MCI_HEADPHONES_CONNECTOR:
               WinPostMsg(hwnd,
                   WM_CONTROL,
                   MPFROM2SHORT(IDD_RB_CONNECTOR1, BN_CLICKED),
                   MPFROMP(TRUE));
               break;
          case MCI_CD_STREAM_CONNECTOR:
               WinPostMsg(hwnd,
                   WM_CONTROL,
                   MPFROM2SHORT(IDD_RB_CONNECTOR2, BN_CLICKED),
                   NULL);
               break;
          }

          // Allow only the possible selections
          WinEnableControl(hwnd,
                           IDD_RB_CONNECTOR1,
                           (BOOL) ptr->capacities.Headphones);
          WinEnableControl(hwnd,
                           IDD_RB_CONNECTOR2,
                           ptr->capacities.Stream);
          break;

     //----------------------------------

     case WM_CONTROL:
          if(SHORT2FROMMP(mp1) == BN_CLICKED) {
               // That's the only event we'll deal with

               // Unselect whichever button is currently
               // selected.  Do this by querying which
               // button is selected
               int current = (int) WinSendDlgItemMsg(hwnd,
                              IDD_RB_CONNECTOR1,
                              BM_QUERYCHECKINDEX,
                              NULL,
                              NULL);
               switch(current) {
               case 1:
                    WinSendDlgItemMsg(hwnd,
                                 IDD_RB_CONNECTOR1,
                                 BM_SETCHECK,
                                 MPARAM(FALSE),
                                 NULL);
                    break;
               case 2:
                    WinSendDlgItemMsg(hwnd,
                                 IDD_RB_CONNECTOR2,
                                 BM_SETCHECK,
                                 MPARAM(FALSE),
                                 NULL);
                    break;
               }

               // Select the appropriate button
               WinSendDlgItemMsg(hwnd,
                  SHORT1FROMMP(mp1),
                  BM_SETCHECK,
                  MPARAM(TRUE),
                  NULL);

               // And set the selected connector
               switch(SHORT1FROMMP(mp1)) {
               case IDD_RB_CONNECTOR1:
                    ptr->Set_Connector(MCI_HEADPHONES_CONNECTOR);
                    break;
               case IDD_RB_CONNECTOR2:
                    ptr->Set_Connector(MCI_CD_STREAM_CONNECTOR);
                    break;
               }
          }
}//switch(msg)

return WinDefDlgProc(hwnd, msg, mp1, mp2);
}// Dlg_Connector()

//---------------------------------------------------------
//---------------------------------------------------------

// This dialog contains two circular sliders that are used
// to set the volume of the left and right channels.
// The volume is updated as the values of the circular
// sliders are modified, so as to provide immediate
// feedback.  To implement this, I simply passed the
// Class_CDROM class object.

MRESULT EXPENTRY Dlg_Volume(HWND hwnd,
                            ULONG msg,
                            MPARAM mp1,
                            MPARAM mp2) {
static Class_CDROM *ptr;
Struct_Volume volume;

switch(msg){
     case WM_INITDLG:
          // Retrieve the pointer to the Class_CDROM object
          ptr = (Class_CDROM *) mp2;
          // and the current volume level
          volume = ptr->Get_Volume();

          // Seemingly all sliders are incompletely
          // defined in resource files.  Let's finish the
          // job here.
          WinSendMsg(WinWindowFromID(hwnd, IDD_CSLIDER_LEFT),
                     CSM_SETRANGE,
                     MPFROMLONG(0),    // minimal value
                     MPFROMLONG(100)); // maximal value
          WinSendMsg(WinWindowFromID(hwnd, IDD_CSLIDER_LEFT),
                     CSM_SETINCREMENT,
                     MPFROMLONG(5),   // increment when using the mouse
                     MPFROMLONG(10)); // number of ticks around the slider
          WinSendMsg(WinWindowFromID(hwnd, IDD_CSLIDER_LEFT),
                     CSM_SETVALUE,
                     MPFROMLONG(volume.left), // set the value
                     NULL);
          WinSendMsg(WinWindowFromID(hwnd, IDD_CSLIDER_RIGHT),
                     CSM_SETRANGE,
                     MPFROMLONG(0),
                     MPFROMLONG(100));
          WinSendMsg(WinWindowFromID(hwnd, IDD_CSLIDER_RIGHT),
                     CSM_SETINCREMENT,
                     MPFROMLONG(5),
                     MPFROMLONG(10));
          WinSendMsg(WinWindowFromID(hwnd, IDD_CSLIDER_RIGHT),
                     CSM_SETVALUE,
                     MPFROMLONG(volume.right),
                     NULL);
          WinSendMsg(hwnd, WM_UPDATEFRAME, 0, 0);
          break;

     //----------------------------------

     case WM_CONTROL:
          if(SHORT2FROMMP(mp1) == CSN_TRACKING) {
               // That's the only event we'll deal with

               // Retrieve the value of the buttons
               WinSendMsg(WinWindowFromID(hwnd, IDD_CSLIDER_LEFT),
                  CSM_QUERYVALUE,
                   &volume.left, // place the value in here
                   0);
               WinSendMsg(WinWindowFromID(hwnd, IDD_CSLIDER_RIGHT),
                  CSM_QUERYVALUE,
                  &volume.right, // place the value in here
                  0);

               // Set the new volume level
               ptr->Set_Volume(volume);
          }
          break;

}//switch(msg)
return WinDefDlgProc(hwnd, msg, mp1, mp2);
}// Dlg_Volume()
